from timemachines.skaters.pmd import pmd_auto
from timemachines.skaters.dlmseasonal import dlm_seasonal
from timemachines.skaters.dlmauto import dlm_auto

SKATERS = [ pmd_auto, dlm_auto, dlm_seasonal ]